function  x=elimination_strategy(BJT,F,V,M,Delta,strategy)

%elimination_strategy: search an "optimal" elimination strategy
%
%x=elimination_strategy(BJT,F,V,M,Delta,strategy) returns the next node to be
%eliminated
%
%INPUT:
% BJT=binary join tree (leaves)
% Delta=guess elimination sequence
% M=bbm
% F=frames
% V=variables
% strategy=type of strategy
%OUTPUT:
% x=next node to be eliminated

nd=length(Delta);
if strategy==0
    if nd>0
        x=Delta(1);
    end
else
    %Calculate the closure of nodes in Delta
    Leaf=[];
    for i=1:nd %for every node in Delta
        Closure(i).c=[];
        d=Delta(i);
        is_leaf=0;
        for j=1:BJT.n %calculate the closure of d
            if isempty(intersect(d,BJT.node(j).dom))==0
                Closure(i).c=union(Closure(i).c,BJT.node(j).dom); %closure of d
                is_leaf=is_leaf+1;
            end
        end
        if is_leaf==1 %if node is a leaf
            Leaf=[Leaf i];
        end
    end

    %STRATEGY
    smin=Inf;
    x=0;
    sfocal=Inf;
    if (strategy==1) %simple strategy

        if isempty(Leaf)==0 %if there is a leaf
            for i=1:length(Leaf)
                %if the cardinality of the closure of node i is min
                if length(Closure(i).c)<smin
                    smin=length(Closure(i).c);
                    x=Delta(i);
                end
            end
        else %as the previous one but in the case no leaves
            for i=1:nd
                if length(Closure(i).c)<smin
                    smin=length(Closure(i).c);
                    x=Delta(i);
                end
            end
        end

    else

        if isempty(Leaf)==0 %if there is a leaf
            for i=1:length(Leaf)
                %if the cardinality of the closure of node i is min
                if prod([F([V([Closure(i).c]).frame]).card])<smin
                    smin=prod([F([V([Closure(i).c]).frame]).card]);
                    x=Delta(i);
                elseif prod([F([V([Closure(i).c]).frame]).card])==smin
                    if prod([M([Closure(i).c]).num_focal])<sfocal
                        sfocal=prod([M([Closure(i).c]).num_focal]);
                        smin=prod([F([V([Closure(i).c]).frame]).card]);
                        x=Delta(i);
                    end
                end
            end
        else %as the previous one but in the case no leaves
            for i=1:nd
                if prod([F([V([Closure(i).c]).frame]).card])<smin
                    smin=prod([F([V([Closure(i).c]).frame]).card]);
                    x=Delta(i);
                elseif prod([F([V([Closure(i).c]).frame]).card])==smin
                    if prod([M([Closure(i).c]).num_focal])<sfocal
                        sfocal=prod([M([Closure(i).c]).num_focal]);
                        smin=prod([F([V([Closure(i).c]).frame]).card]);
                        x=Delta(i);
                    end
                end
            end
        end

    end
end
