function [BJT, M, nm, N, Del_order]=construct_binary_join_tree(BJT, Npsi, Delta, M, nm, F, V, strategy)

% construct_binary_join_tree: build a binary join tree
% 
% [BJT, M, nm, N, Del_order]=construct_binary_join_tree(BJT, Npsi, Delta, M, nm, F, V, strategy)
% returns the binary join tree.
%
%INPUT:
% BJT=binary join tree (leaves)
% Npsi=sequence of bbm (leaves of the tree)
% Delta=guess elimination sequence 
% M=bbm
% nm=number of bbm
% F=frames
% V=variables
% strategy=type of strategy to find an elimination sequence
%OUTPUT:
% BJT=binary join tree
% M=bbm
% nm=number of bbm
% N=sequence of nodes of the tree
% Del_order="optimal" elimination sequence


N=[]; %sequence of nodes of the tree
Deltac=[]; %contains the eliminated variable
root=0; %root of the BJT
Del_order=[]; %deletion order
while root==0
    if isempty(Delta)==1
        Nx=Npsi;
    else
        x=elimination_strategy(BJT,F,V,M,Delta,strategy); %search the next variable to be eliminated 
        Del_order=[Del_order x]; 
        Nx=[];
        for i=1:length(Npsi)
            if isempty(find(M(Npsi(i)).v==x))==0 %find the node with a domain containing x
                Nx=[Nx Npsi(i)];
            end
        end
    end
    while length(Nx)>1
        %create a new node in the BJT
        BJT.n=BJT.n+1; 
        BJT.node(BJT.n).father=0;
        n12=Nx(1:2);
        BJT.node(n12(1)).father=BJT.n;
        BJT.node(n12(2)).father=BJT.n;
        BJT.node(BJT.n).lson=n12(1);
        BJT.node(BJT.n).rson=n12(2);
        BJT.node(BJT.n).dom=sort(setdiff(union(BJT.node(n12(1)).dom,BJT.node(n12(2)).dom),Deltac));
       
        %create a new element in M for the new node 
        nm=nm+1;
        M(nm).v=BJT.node(BJT.n).dom;%variables
        num_v=size(M(nm).v,2); %num variables
        M(nm).nv=num_v;
        %calculate the cardinality of the domain of the new node
        card=[];
        pcard=[];
        for i=1:num_v
            card=[card F(V(M(nm).v(i)).frame).card];
            pcard(i).el=1:card(i);
        end
        M(nm).cartprod=card;%% cardinality of single frames
        M(nm).ne=prod(card);%%cardinality of joint set
        M(nm).matrix=cartprod(pcard(1:num_v).el);%%cartesian product
        M(nm).num_focal=0; %1;%%num focal elements
        M(nm).foc(1).elem=[]; %[1:M(nm).ne]; %universe
        M(nm).foc(1).mass=[]; %1;

        %updating for the next step
        Nx=union(setdiff(Nx,[n12(1) n12(2)]),BJT.n);
        N=union(N,[n12(1) n12(2)]);
    end
    n=Nx;
    if isempty(Delta)==1
        root=n;
    else
        fd=find(Delta==x);
        sd=length(Delta);
        Delta=Delta(setdiff([1:sd],fd));
        Deltac=union(Deltac,x);
        Npsiold=Npsi;
        Npsi=[]; %
        for i=1:length(Npsiold)
            if isempty(find(M(Npsiold(i)).v==x))==1 %search the nodes with domain that doesnt contain x
                Npsi=[Npsi Npsiold(i)];
            end
        end
        Npsi=[Npsi, n];
    end
end
N=[N n];
BJT.root=root;