function [Mroot, M, Ms]=inward(BJT,M,nm,Nn,F,V,D,updated_inputs,stage,Ms)

%inward: calculate the inward propagation phase 
%
%[Mroot, M, Ms]=inward(BJT,M,nm,Nn,F,V,D,updated_inputs,stage,Ms) returns the
%bbm for the decision variable, Mroot, for every node in the BJT, M, and
%the structure Ms that is the bbm after the elimination of the variables
%of not interest.
%
%
%INPUT: 
% BJT: binary join tree
% M: bbm
% nm: nume of bbm
% Nn:
% F: frames
% V: variables
% D: decision variable
% updated_inputs: vector containing the leaves of the tree that are updated from new information
% stage: is 1 in the initialization phase 0 otherwise
% Ms: bbm after the elimination of the variables
%OUTPUT:
% Mroot: belief of the decision variable
% M: belief of the nodes in the BJT
% Ms: belief of the nodes in the BJT after the elimination of the variables
%     of not interest.


%init Leaves
Leaves=[];
for i=1:BJT.n
    if BJT.node(i).rson==0 & BJT.node(i).lson==0 %if node is a leaf
        Leaves=[Leaves i];
    end
end

%init nextN
nextN=[];
if stage==0
    for i=1:BJT.n
        if isempty(find(Leaves==i))==1 %if node is not a leaf
            if isempty(find((BJT.node(i).rson==Leaves)>0))==0 & isempty(find((BJT.node(i).lson==Leaves)>0))==0  %M(i).num_focal==0 & M(BJT.node(i).lson).num_focal>0 & M(BJT.node(i).rson).num_focal>0
                nextN=[nextN i];
            end
        end
    end
    %initialization of the bbm
    Ms=M;
    %var elimination
    sl=size(Leaves,2);
    for i=1:sl
        delta=setdiff(BJT.node(i).dom,BJT.node(BJT.node(i).father).dom);
        Ms(i)=var_elim(i,M,delta,F,V);
    end
else
    for i=1:BJT.n
        if isempty(find(Leaves==i))==1 %if node is not a leaf
            if (isempty(find((BJT.node(i).rson==updated_inputs)>0))==0 | isempty(find((BJT.node(i).lson==updated_inputs)>0))==0) %M(i).num_focal==0 & M(BJT.node(i).lson).num_focal>0 & M(BJT.node(i).rson).num_focal>0
                nextN=[nextN i];
            end
        end
    end
end


visitN=[];
while isempty(nextN)==0
    n=nextN(1); %next node to be processed
    visitN=[visitN n]; %visited nodes
    nextN=setdiff(nextN,nextN(1)); %updating nodes have to be visited
    M(n)=val_prod(Ms(BJT.node(n).lson),Ms(BJT.node(n).rson),BJT.node(n).dom,F,V); %Dempster-Shafer combination rule
    if n<BJT.root
        faths=BJT.node(BJT.node(n).father).dom; %father
    else
        faths=D(1).cod; %decision variavble
    end
    delta=setdiff(BJT.node(n).dom,faths); %variables to be eliminated
    Ms(n)=var_elim(n,M,delta,F,V); %elimination
    %update nextN
    if stage==0
        for i=1:BJT.n
            if isempty(find(Leaves==i))==1 & isempty(find(nextN==i))==1 & isempty(find(visitN==i)>0)==1 %if node is not a leaf or if it is not in nextN or if it has not been visited
                %if isempty(find(i==visitN)) &
                if ((isempty(find(BJT.node(i).lson==visitN))==0 | isempty(find(BJT.node(i).lson==Leaves))==0) & (isempty(find(BJT.node(i).rson==Leaves))==0 | isempty(find(BJT.node(i).rson==visitN))==0) )
                    nextN=[nextN i];
                end
            end
        end
    else
        for i=1:BJT.n
            if isempty(find(updated_inputs==i))==1 & isempty(find(nextN==i))==1 & isempty(find(visitN==i)>0)==1 %if node is not in updated_inputs or if it is not in nextN or if it has not been visited
                %if isempty(find(i==visitN)) &
                    if ((isempty(find(BJT.node(i).lson==visitN))==0) | (isempty(find(BJT.node(i).rson==visitN))==0) )
                    nextN=[nextN i];
                end
            end
        end
    end
end

%bbm of the decision variable
Mroot=Ms(BJT.root);